import numpy as np
from astropy.nddata import block_reduce
import sys
import os
import math
from adari_core.report import AdariReportBase
from adari_core.plots.panel import Panel
from adari_core.plots.points import LinePlot
from adari_core.plots.text import TextPlot
from adari_core.utils.utils import fetch_kw_or_default
from adari_core.utils.utils import pretty_print_units
from adari_core.utils.clipping import clipping_percentile

from . import EspressoReportMixin


class EspressoScienceReport(EspressoReportMixin, AdariReportBase):
    def __init__(self):
        super().__init__("espresso_science")
        self.stacked = False

    def parse_sof(self):
        sci_spec = []
        stacked_spec = None

        for filename, catg in self.inputs:
            if catg == "S1D_FINAL_A":
                sci_spec.append(filename)
            if catg == "ESOTK_SPECTRUM_IDP_FORMAT":
                stacked_spec = filename

        file_lists = []
        if stacked_spec is not None:
            file_lists.append(
                {
                    "science": stacked_spec,
                }
            )
            single_count = 0
            for single in sci_spec :
                file_lists.append(
                    {
                        "single%03i"%single_count: single,
                    }
                )
                single_count += 1
            self.stacked = True
        elif len(sci_spec) > 0:
            file_lists.append(
                {
                    "science": sci_spec[0],
                }
            )
        return file_lists

    def generate_panels(self, **kwargs):
        panels = {}

        science =  self.hdus[0]["science"]
        first_single =  self.hdus[0]["science"]
        if self.stacked :
            sorted_raw_name = sorted(
                    [v for v in self.hdus if any("single" in s for s in v.keys())], # Get the frames with single
                    key=lambda x: next(iter(x.values()), None)[0].header.get("ESO PRO REC1 RAW1 NAME", ""),  # Order by raw file name
            )
            first_single = list(sorted_raw_name[0].values())[0] # Input with first raw file name in alphabetical order
        science_fname = os.path.basename(str(science.filename()))

        # Locate the science with the lowest and highest average
        if self.stacked:
            sorted_avg_flux = sorted(
                    [v for v in self.hdus if any("single" in s for s in v.keys())], # Get the frames with single
                    key=lambda x: next(iter(x.values()), None)[0].header.get("ESO QC SPEC AVG", -999.0),  # Order by average flux
            )
            lowest_flux_science = list(sorted_avg_flux[0].values())[0] # Get the science with the lowest average flux
            highest_flux_science = list(sorted_avg_flux[-1].values())[0] # Get the science with the highest average flux

        self.wavelength = science["SPECTRUM"].data["WAVE"][0] / 10
        self.wavelength_unit = science["SPECTRUM"].header["TUNIT1"]
        if "FLUX" in science["SPECTRUM"].data.columns.names: 
            self.flux = science["SPECTRUM"].data["FLUX"][0]
        else:
            self.flux = science["SPECTRUM"].data["FLUX_REDUCED"][0]
        self.flux *= 1e16
        self.flux_unit = pretty_print_units(science["SPECTRUM"].header["TUNIT2"])
        self.snr = science["SPECTRUM"].data["SNR"][0]
        self.qual = science["SPECTRUM"].data["QUAL"][0]

        if self.stacked :
            p = Panel(6, 7, height_ratios=[1.5, 5, 4, 1, 1, 3, 1.5],  y_stretch=0.6)
        else :
            p = Panel(6, 6, height_ratios=[1, 5, 1, 1, 3, 1],  y_stretch=0.7)

        specplot = LinePlot(title="",
                           )
        specplot.add_data(d=[self.wavelength, self.flux], 
                          color="royalblue", 
                          label="Flux",
                          linewidth = 0.4)
        ins_mode = fetch_kw_or_default(science["PRIMARY"], "ESO INS MODE", default="N/A")
        if ins_mode == 'MULTIMR':
            binning = 0.4 # in nm
        else:
            binning = 0.2 # in nm
        binning_pix = int(len(self.wavelength) / ((self.wavelength[-1] - self.wavelength[0]) / binning))
        wave_binned = block_reduce(self.wavelength, binning_pix, func=np.mean)
        flux_binned = block_reduce(self.flux, binning_pix, func=np.mean)
        specplot.add_data(d=[wave_binned, flux_binned],
                          color="darkorange",
                          label="Averaged flux",
                          linewidth = 1.5)
        spectral_lines = {"Hα": 656.280,
                          "Hβ": 486.132,
                          "Hγ": 434.046,
                          "Hδ": 410.173,
                          "Hε": 397.007,
                          "H10": 379.790,
                          "H17": 369.715,
                          "Na D": 589.29,
                          "P7": 1004.94,
                          "P8": 954.598,
                          "P9": 922.902,
                          "P10": 901.491,
                          "P17": 846.72
                         }
        espresso_lines = ['Hα','Hβ','Hγ','Hδ','Hε','Na D']
        wave_lines = [var for (key,var) in spectral_lines.items() if (key in espresso_lines)]
        specplot.add_data(d=wave_lines,
                          color="limegreen",
                          label="H lines",
                          linewidth = 1.8,
                          vline=True,
                          linestyle='dashed')
        specplot.x_minor_ticks = 0
        specplot.x_label = f"Wavelength (nm)"
        specplot.y_label = f"{self.flux_unit}"
        specplot.set_xlim(self.wavelength[0], self.wavelength[-1])
        specplot.set_ylim(0, clipping_percentile(self.flux, 98)[1] * 1.2)
        for name, wave in spectral_lines.items() :
            if name in espresso_lines :
                specplot.add_text(text=name, x=wave, y=specplot.y_max*0.98,  rotation=90, horizontalalignment='right', verticalalignment='top', fontsize=12)

        p.assign_plot(specplot, 0, 1, xext=6)

        if self.stacked:
            lowhighplot = LinePlot(title="")
            low_wave_binned = block_reduce(lowest_flux_science["SPECTRUM"].data["WAVE"][0] / 10, binning_pix, func=np.mean)
            if "FLUX" in lowest_flux_science["SPECTRUM"].data.columns.names:
                low_flux_binned = block_reduce(lowest_flux_science["SPECTRUM"].data["FLUX"][0], binning_pix, func=np.mean) * 1e16
            else:
                low_flux_binned = block_reduce(lowest_flux_science["SPECTRUM"].data["FLUX_REDUCED"][0], binning_pix, func=np.mean) * 1e16
            high_wave_binned = block_reduce(highest_flux_science["SPECTRUM"].data["WAVE"][0] / 10, binning_pix, func=np.mean)
            if "FLUX" in highest_flux_science["SPECTRUM"].data.columns.names:
                high_flux_binned = block_reduce(highest_flux_science["SPECTRUM"].data["FLUX"][0], binning_pix, func=np.mean) * 1e16
            else:
                high_flux_binned = block_reduce(highest_flux_science["SPECTRUM"].data["FLUX_REDUCED"][0], binning_pix, func=np.mean) * 1e16
            lowhighplot.add_data(d=[low_wave_binned, low_flux_binned],
                          color="darkblue",
                          label="Lowest flux",
                          linewidth = 1.5)
            lowhighplot.add_data(d=[high_wave_binned, high_flux_binned],
                          color="darkgreen",
                          label="Highest flux",
                          linewidth = 1.5)
            lowhighplot.add_data(d=[wave_binned, flux_binned],
                          color="darkorange",
                          label="Averaged flux",
                          linewidth = 1.5)
            lowhighplot.x_minor_ticks = 0
            lowhighplot.set_xlim(wave_binned[0], wave_binned[-1])
            lowhighplot.set_ylim(0, clipping_percentile(high_flux_binned, 98)[1] * 1.2)
            lowhighplot.x_label = f"Wavelength (nm)"
            lowhighplot.y_label = f"{self.flux_unit}"

            p.assign_plot(lowhighplot, 0, 2, xext=6)



        snrplot = LinePlot(title="",
                           legend = False,
                           )
        snrplot.add_data(d=[self.wavelength, self.snr],
                         color = 'cyan',
                         label="ignore",
                         linewidth = 0.4)
        snrplot.x_minor_ticks = 0
        snrplot.x_label = f"Wavelength (nm)"
        snrplot.y_label = "S/N"
        snrplot.set_xlim(self.wavelength[0], self.wavelength[-1])

        p.assign_plot(snrplot, 0, 2 + self.stacked, xext=6)

        qualplot = LinePlot(title="",
                            legend = False,
                            )
        qualplot.add_data(d=[self.wavelength, self.qual],
                          color = 'lightgreen',
                          label="Q")
        qualplot.x_label = f"Wavelength (nm)"
        qualplot.y_label = "Q"
        qualplot.x_minor_ticks = 0
        qualplot.set_xlim(self.wavelength[0], self.wavelength[-1])
        qualplot.set_ylim(0, clipping_percentile(self.qual, 98)[1] * 1.2)

        p.assign_plot(qualplot, 0, 3 + self.stacked, xext=6)
        # Title Plot

        # Text Plot
        vspace = 0.4
        t0 = TextPlot(columns=1, v_space=vspace)
        col0 = (
                str(fetch_kw_or_default(science["PRIMARY"], "INSTRUME", default="N/A"))+" science product preview",
                "Product: " + str(fetch_kw_or_default(science["PRIMARY"], "ESO PRO CATG", default="N/A")),
                "Raw file: " + str(fetch_kw_or_default(first_single["PRIMARY"], "ESO PRO REC1 RAW1 NAME", default="N/A")),
                "MJD-OBS: " + str(fetch_kw_or_default(science["PRIMARY"], "MJD-OBS", default="N/A")),
        )
        t0.add_data(col0, fontsize=13)
        p.assign_plot(t0, 0, 0, xext=1)

        t1 = TextPlot(columns=1, v_space=vspace)
        col1 = (
                "Target: " + str(fetch_kw_or_default(science["PRIMARY"], "OBJECT", default="N/A")),
                "OB ID: " + str(fetch_kw_or_default(science["PRIMARY"], "ESO OBS ID", default="N/A")),
                "OB NAME: " + str(fetch_kw_or_default(science["PRIMARY"], "ESO OBS NAME", default="N/A")),
                "TPL ID: " + str(fetch_kw_or_default(science["PRIMARY"], "ESO TPL ID", default="N/A")),
                "RUN ID: " + str(fetch_kw_or_default(science["PRIMARY"], "ESO OBS PROG ID", default="N/A")),
        )
        t1.add_data(col1, fontsize=13)
        p.assign_plot(t1, 2, 0, xext=1)

        t2 = TextPlot(columns=1, v_space=vspace)
        spec_type = fetch_kw_or_default(science["PRIMARY"], "ESO OCS OBJ SP TYPE", default="N/A")
        if self.stacked :
            fibre_b = str(fetch_kw_or_default(science["PRIMARY"], "ESO STACK QC FIB B1", default="N/A"))
        else :
            fibre_b = str(fetch_kw_or_default(science["PRIMARY"], "ESO QC FIB B", default="N/A"))
        col2 = (
                "INS MODE: " + str(fetch_kw_or_default(science["PRIMARY"], "ESO INS MODE", default="N/A")),
                "Binning: " + str(fetch_kw_or_default(science["PRIMARY"], "ESO DET BINX", default="N/A")) + " x " +
                    str(fetch_kw_or_default(science["PRIMARY"], "ESO DET BINY", default="N/A")),
                "Telescope: " + str(fetch_kw_or_default(science["PRIMARY"], "TELESCOP", default="N/A")).replace("ESO-VLT-","").replace("U", "UT"),
                "Fibre B: " + fibre_b,
                "Spec type: " + spec_type,
        )
        t2.add_data(col2, fontsize=13)
        p.assign_plot(t2, 4, 0, xext=1)

        hgspecplot = LinePlot(title="Hγ",
                           legend = False,
                           )
        hgspecplot.add_data(d=[self.wavelength, self.flux], 
                          color="blue", 
                          label="flux",
                          linewidth = 0.4)
        hgspecplot.x_label = f"Wavelength (nm)"
        hgspecplot.y_label = f"{self.flux_unit}"
        hgspecplot.set_xlim(spectral_lines['Hγ'] - 1, spectral_lines['Hγ'] + 1)
        flux_selected = self.flux[abs(self.wavelength - spectral_lines['Hγ']) < 1]
        hgspecplot.set_ylim(0, 1.2 * clipping_percentile(flux_selected, 99)[1])
        hgspecplot.x_major_ticks = 0.5


        p.assign_plot(hgspecplot, 0, 4 + self.stacked, xext=2)

        nadspecplot = LinePlot(title="Na D",
                           legend = False,
                           )
        nadspecplot.add_data(d=[self.wavelength, self.flux], 
                          color="blue", 
                          label="flux",
                          linewidth = 0.4)
        nadspecplot.x_label = f"Wavelength (nm)"
        nadspecplot.y_label = f"{self.flux_unit}"
        nadspecplot.set_xlim(spectral_lines['Na D'] - 1, spectral_lines['Na D'] + 1)
        flux_selected = self.flux[abs(self.wavelength - spectral_lines['Na D']) < 1]
        nadspecplot.set_ylim(0, 1.2 * clipping_percentile(flux_selected, 99)[1])
        nadspecplot.x_major_ticks = 0.5


        p.assign_plot(nadspecplot, 2, 4 + self.stacked, xext=2)

        haspecplot = LinePlot(title="Hα",
                           legend = False,
                           )
        haspecplot.add_data(d=[self.wavelength, self.flux], 
                          color="blue", 
                          label="flux",
                          linewidth = 0.4)
        haspecplot.x_label = f"Wavelength (nm)"
        haspecplot.y_label = f"{self.flux_unit}"
        haspecplot.set_xlim(spectral_lines['Hα'] - 1, spectral_lines['Hα'] + 1)
        flux_selected = self.flux[abs(self.wavelength - spectral_lines['Hα']) < 1]
        haspecplot.set_ylim(0, 1.2 * clipping_percentile(flux_selected, 99)[1])
        haspecplot.x_major_ticks = 0.5
        

        p.assign_plot(haspecplot, 4, 4 + self.stacked, xext=2)

        # Text Plot
        vspace = 0.4
        t4 = TextPlot(columns=1, v_space=vspace)
        col4 = ()
        if self.stacked:
            col4 += ("Total exp. time [s]: " + "%.1f" % fetch_kw_or_default(science["PRIMARY"], "TEXPTIME", default="N/A"),
                     "N exposures (sqrt N): " + "%i (%0.1f)" % (fetch_kw_or_default(science["PRIMARY"], "NCOMBINE", default="N/A"),
                        math.sqrt(fetch_kw_or_default(science["PRIMARY"], "NCOMBINE", default="N/A"))),
                     "Avg. contribution: " + "%.1f" % fetch_kw_or_default(science["PRIMARY"], "ESO QC CONTRIB AVG", default="N/A"),
                     "Avg. S/N stack: " + "%.1f" % fetch_kw_or_default(science["PRIMARY"], "ESO QC SNR AVG", default="N/A"),
            )
        else:
            col4 +=("Exp. time [s]: " + "%.1f" % fetch_kw_or_default(science["PRIMARY"], "TEXPTIME", default="N/A"),
                    "N exposures: " + "%i" % fetch_kw_or_default(science["PRIMARY"], "NCOMBINE", default="N/A"),
                    "S/N average: " + "%.1f" % fetch_kw_or_default(science["PRIMARY"], "ESO QC SNR AVG", default="N/A"),
                    "Lambda bin [nm]: " + "%.5f" % fetch_kw_or_default(science["PRIMARY"], "SPEC_BIN", default="N/A"),
            )
        t4.add_data(col4, fontsize=13)
        p.assign_plot(t4, 0, 5 + self.stacked, xext=1)

        t5 = TextPlot(columns=1, v_space=vspace)
        col5 = ()
        if self.stacked:
            col5 += (
                    "Seeing: " + "%.2f" % fetch_kw_or_default(science["PRIMARY"], "ESO QC IA FWHM AVG", default="N/A"),
                    "First N pix sat: " + "%i" % fetch_kw_or_default(first_single["PRIMARY"], "ESO QC SAT NB", default="N/A"),
                    "Avg. S/N all single: " + "%.1f" % (fetch_kw_or_default(science["PRIMARY"], "ESO QC SNR AVG", default=0) /
                         fetch_kw_or_default(science["PRIMARY"], "ESO QC SNR RATIO", 1)),
                    "Avg. S/N improvement: " + "%.1f" % fetch_kw_or_default(science["PRIMARY"], "ESO QC SNR RATIO", "N/A"),
            )
        else:
            col5 += (
                    "Seeing: " + "%.2f" % fetch_kw_or_default(science["PRIMARY"], "ESO QC IA FWHM AVG", default="N/A"),
                    "N pix sat: " + "%i" % fetch_kw_or_default(first_single["PRIMARY"], "ESO QC SAT NB", default="N/A"),
                    "Delta time wave A [d]: " + "%.2f" % fetch_kw_or_default(first_single["PRIMARY"], "ESO QC DELTA TIME WAVE A", default="N/A"),
            )
        t5.add_data(col5, fontsize=13)
        p.assign_plot(t5, 2, 5 + self.stacked, xext=1)

        t6 = TextPlot(columns=1, v_space=vspace)
        if(spec_type[0] in ['F', 'G', 'K', 'M']) :
                ccf_rv = "%.6f" % fetch_kw_or_default(first_single["PRIMARY"], "ESO QC CCF RV", default="N/A")
                ccf_rv_err = "%.6f" % fetch_kw_or_default(first_single["PRIMARY"], "ESO QC CCF RV ERROR", default="N/A")
                ccf_mask = fetch_kw_or_default(first_single["PRIMARY"], "ESO QC CCF MASk", default="N/A")
        else:
                ccf_rv = "N/A"
                ccf_rv_err = "N/A"
                ccf_mask = "N/A"
        if self.stacked:
            col6 = (
                    "First Delta time wave A [d]: " + "%.2f" % fetch_kw_or_default(first_single["PRIMARY"], "ESO QC DELTA TIME WAVE A", default="N/A"),
                    "First CCF RV [km/s]: " + ccf_rv,
                    "First CCF RV ERROR [km/s]: " + ccf_rv_err,
                    "First CCF MASK: " + ccf_mask,
            )
        else:
            col6 = (
                    "CCF RV [km/s]: " + ccf_rv,
                    "CCF RV ERROR [km/s]: " + ccf_rv_err,
                    "CCF MASK: " + ccf_mask,
            )
        t6.add_data(col6, fontsize=13)
        p.assign_plot(t6, 4, 5 + self.stacked, xext=1)

        addme = {
            "report_name": f"ESPRESSO_{str(science_fname).removesuffix('.fits').lower()}",
            "report_description": f"Science panel",
            "report_tags": [],
            "report_prodcatg" : "ANCILLARY.PREVIEW",
        }

        panels[p] = addme

        return panels


rep = EspressoScienceReport()
